"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRpcServer = void 0;
const tslib_1 = require("tslib");
const JsonRpcRxServer_1 = require("../json-rx/JsonRpcRxServer");
const util_1 = require("../json-rx/util");
class JsonRpcServer {
    constructor(params) {
        this.rx = new JsonRpcRxServer_1.JsonRpcRxServer(params);
        this.strict = params.strict || false;
    }
    formatError(error, id = null) {
        const message = {
            id,
            error,
        };
        if (this.strict)
            message.jsonrpc = '2.0';
        return message;
    }
    onMessage(ctx, message) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let id = null;
            try {
                if (!message || (typeof message !== 'object') || util_1.isArray(message))
                    return this.formatError({ message: 'Invalid message.', code: 0 }, id);
                const isNotification = (typeof message.id !== 'number') && (typeof message.id !== 'string');
                if (!isNotification)
                    id = message.id;
                if (message.method === undefined)
                    return this.formatError({ message: 'Method not specified.', code: 0 }, id);
                if (typeof message.method !== 'string')
                    return this.formatError({ message: 'Invalid method.', code: 0 }, id);
                if (this.strict && message.jsonrpc !== '2.0')
                    return this.formatError({ message: 'Only JSON-RPC version 2.0 is supported.', code: 0 }, id);
                if (isNotification) {
                    this.rx.onMessage(ctx, [message.method, message.params]);
                    return null;
                }
                const result = yield this.rx.onMessage(ctx, [1, message.method, message.params]);
                if (!result)
                    return null;
                const [type] = result;
                if (type === -1)
                    throw result[2];
                const responseMessage = {
                    id,
                    result: result[2],
                };
                if (this.strict)
                    responseMessage.jsonrpc = '2.0';
                return responseMessage;
            }
            catch (error) {
                const rpcError = {
                    code: (!!error && (typeof error === 'object') && (typeof error.code === 'number'))
                        ? error.code
                        : 0,
                    message: error instanceof Error
                        ? error.message
                        : (!!error && (typeof error === 'object'))
                            ? String(error.message)
                            : String(error),
                };
                if (!!error && (typeof error === 'object') && (error.data !== undefined))
                    rpcError.data = error.data;
                return this.formatError(rpcError, id);
            }
        });
    }
}
exports.JsonRpcServer = JsonRpcServer;
