"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRxClient = void 0;
const rxjs_1 = require("rxjs");
const util_1 = require("./util");
const TimedQueue_1 = require("./TimedQueue");
class JsonRxClient {
    constructor({ send, bufferSize = 100, bufferTime = 10 }) {
        this.cnt = 1;
        this.observers = new Map();
        this.buffer = new TimedQueue_1.TimedQueue();
        this.buffer.itemLimit = bufferSize;
        this.buffer.timeLimit = bufferTime;
        this.buffer.onFlush = (messages) => {
            send(messages.length === 1 ? messages[0] : messages);
        };
    }
    sendSubscribe(message) {
        this.buffer.push(message);
    }
    sendUnsubscribe(id) {
        this.buffer.push([-3, id]);
    }
    onData(message) {
        const [, id, payload] = message;
        util_1.assertId(id);
        const observer = this.observers.get(id);
        if (!observer)
            return;
        observer.observer.next(payload);
    }
    onComplete(message) {
        const [, id, payload] = message;
        util_1.assertId(id);
        const observer = this.observers.get(id);
        if (!observer)
            return;
        observer.completed = true;
        if (payload !== undefined)
            observer.observer.next(payload);
        observer.observer.complete();
    }
    onError(message) {
        const [, id, error] = message;
        util_1.assertId(id);
        const observer = this.observers.get(id);
        if (!observer)
            return;
        observer.completed = true;
        observer.observer.error(error);
    }
    onMessage(message) {
        if (!util_1.isArray(message))
            throw new Error('Invalid message');
        const [type] = message;
        if (util_1.isArray(type)) {
            for (let i = 0; i < message.length; i++)
                this.onMessage(message[i]);
            return;
        }
        if (type === 0)
            return this.onComplete(message);
        if (type === -1)
            return this.onError(message);
        if (type === -2)
            return this.onData(message);
        throw new Error('Invalid message');
    }
    call(method, payload) {
        const id = this.cnt++;
        const observable = new rxjs_1.Observable((observer) => {
            const entry = { observer };
            this.observers.set(id, entry);
            return () => {
                this.observers.delete(id);
                if (!entry.completed)
                    this.sendUnsubscribe(id);
            };
        });
        this.sendSubscribe([id, method, payload]);
        return observable;
    }
    notify(name, payload) {
        const message = payload !== undefined ? [name, payload] : [name];
        this.buffer.push(message);
    }
    stop() {
        this.buffer.onFlush = (message) => { };
        for (const { observer } of this.observers.values())
            observer.complete();
    }
}
exports.JsonRxClient = JsonRxClient;
