"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRxServer = void 0;
const json_schema_serializer_1 = require("json-schema-serializer");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const util_1 = require("./util");
const TimedQueue_1 = require("./TimedQueue");
const formatError = (error) => {
    let json = '{"message":' + json_schema_serializer_1.asString(error.message);
    if (typeof error.status === 'number')
        json += ',"status":' + error.status;
    if (typeof error.code === 'string')
        json += ',"code":' + json_schema_serializer_1.asString(error.code);
    if (typeof error.errno === 'number')
        json += ',"errno":' + error.errno;
    if (typeof error.errorId === 'string')
        json += ',"errorId":' + json_schema_serializer_1.asString(error.errorId);
    return json + '}';
};
class JsonRxServer {
    constructor({ send, call, notify, maxActiveSubscriptions = 30, bufferSize = 10, bufferTime = 1 }) {
        this.active = new Map();
        this.call = call;
        this.notify = notify;
        this.maxActiveSubscriptions = maxActiveSubscriptions;
        if (bufferTime) {
            const buffer = new TimedQueue_1.TimedQueue();
            buffer.itemLimit = bufferSize;
            buffer.timeLimit = bufferTime;
            buffer.onFlush = (messages) => {
                send(messages.length === 1 ? messages[0] : '[' + messages.join(',') + ']');
            };
            this.send = message => buffer.push(message);
        }
        else {
            this.send = send;
        }
    }
    sendError(id, error) {
        const message = '[-1,' + id + ',' + JSON.stringify(error) + ']';
        this.send(message);
    }
    onSubscribe(message, ctx) {
        const [id, name, payload] = message;
        util_1.assertId(id);
        util_1.assertName(name);
        try {
            const active = this.active.get(id);
            if (active) {
                this.active.delete(id);
                active.unsubscribe();
                this.send('[-1,' + id + ',{"message":"ID already active."}]');
                return;
            }
            if (this.active.size >= this.maxActiveSubscriptions)
                return this.sendError(id, { message: 'Too many subscriptions.' });
            const callResult = this.call(name, payload, ctx);
            const observable = rxjs_1.isObservable(callResult)
                ? callResult
                : rxjs_1.from(callResult).pipe(operators_1.mergeMap(value => rxjs_1.isObservable(value) ? value : rxjs_1.of(value)));
            const ref = { buffer: [] };
            let done = false;
            const subscription = observable.subscribe((data) => {
                if (data === undefined)
                    return;
                ref.buffer.push(data);
                util_1.microtask(() => {
                    if (!ref.buffer.length)
                        return;
                    try {
                        for (const payload of ref.buffer)
                            this.send('[-2,' + id + ',' + payload + ']');
                    }
                    finally {
                        ref.buffer = [];
                    }
                });
            }, (error) => {
                done = true;
                this.active.delete(id);
                let message;
                if (error instanceof Error) {
                    message = '[-1,' + id + ',' + formatError(error) + ']';
                }
                else {
                    message = '[-1,' + id + ',' + JSON.stringify(error) + ']';
                }
                this.send(message);
            }, () => {
                done = true;
                this.active.delete(id);
                try {
                    if (!ref.buffer.length)
                        return this.send(`[0,${id}]`);
                    const last = ref.buffer.length - 1;
                    for (let i = 0; i <= last; i++) {
                        const isLast = i === last;
                        let message;
                        if (isLast) {
                            message = '[0,' + id + ',' + ref.buffer[i] + ']';
                        }
                        else {
                            message = '[-2,' + id + ',' + ref.buffer[i] + ']';
                        }
                        this.send(message);
                    }
                }
                finally {
                    ref.buffer = [];
                }
            });
            if (!done)
                this.active.set(id, subscription);
        }
        catch (error) {
            this.sendError(id, error instanceof Error ? { message: error.message } : error);
        }
    }
    onUnsubscribe(message) {
        const [, id] = message;
        util_1.assertId(id);
        const subscription = this.active.get(id);
        if (!subscription)
            return;
        this.active.delete(id);
        subscription.unsubscribe();
    }
    onNotification(message, ctx) {
        const [name, payload] = message;
        util_1.assertName(name);
        this.notify(name, payload, ctx);
    }
    onMessage(message, ctx) {
        if (!util_1.isArray(message))
            throw new Error('Invalid message');
        const [one] = message;
        if (util_1.isArray(one)) {
            for (let i = 0; i < message.length; i++)
                this.onMessage(message[i], ctx);
            return;
        }
        if (typeof one === 'string')
            return this.onNotification(message, ctx);
        if (one > 0)
            return this.onSubscribe(message, ctx);
        if (one === -3)
            return this.onUnsubscribe(message);
        throw new Error('Invalid message');
    }
    stop() {
        this.send = (message) => { };
        for (const sub of this.active.values()) {
            sub.unsubscribe();
        }
    }
}
exports.JsonRxServer = JsonRxServer;
