import { JsonRpcRxServer } from '../json-rx/JsonRpcRxServer';
import { isArray } from '../json-rx/util';
export class JsonRpcServer {
    constructor(params) {
        this.rx = new JsonRpcRxServer(params);
        this.strict = params.strict || false;
    }
    formatError(error, id = null) {
        const message = {
            id,
            error,
        };
        if (this.strict)
            message.jsonrpc = '2.0';
        return message;
    }
    async onMessage(ctx, message) {
        let id = null;
        try {
            if (!message || (typeof message !== 'object') || isArray(message))
                return this.formatError({ message: 'Invalid message.', code: 0 }, id);
            const isNotification = (typeof message.id !== 'number') && (typeof message.id !== 'string');
            if (!isNotification)
                id = message.id;
            if (message.method === undefined)
                return this.formatError({ message: 'Method not specified.', code: 0 }, id);
            if (typeof message.method !== 'string')
                return this.formatError({ message: 'Invalid method.', code: 0 }, id);
            if (this.strict && message.jsonrpc !== '2.0')
                return this.formatError({ message: 'Only JSON-RPC version 2.0 is supported.', code: 0 }, id);
            if (isNotification) {
                this.rx.onMessage(ctx, [message.method, message.params]);
                return null;
            }
            const result = await this.rx.onMessage(ctx, [1, message.method, message.params]);
            if (!result)
                return null;
            const [type] = result;
            if (type === -1)
                throw result[2];
            const responseMessage = {
                id,
                result: result[2],
            };
            if (this.strict)
                responseMessage.jsonrpc = '2.0';
            return responseMessage;
        }
        catch (error) {
            const rpcError = {
                code: (!!error && (typeof error === 'object') && (typeof error.code === 'number'))
                    ? error.code
                    : 0,
                message: error instanceof Error
                    ? error.message
                    : (!!error && (typeof error === 'object'))
                        ? String(error.message)
                        : String(error),
            };
            if (!!error && (typeof error === 'object') && (error.data !== undefined))
                rpcError.data = error.data;
            return this.formatError(rpcError, id);
        }
    }
}
